using System;

namespace Resistance
{
	public class OverloadEventArgs : System.EventArgs 
	{
		bool overload;

		public OverloadEventArgs (bool overload)
		{
			this.overload = overload;
		}

		public bool Overload 
		{
			get { return overload; }
		}
	}	// end class


	// To implement a event you need to declare a delegate.
	// A delegate declares and gives a name to a method signature.

	public delegate void OverloadEventHandler(object sender, OverloadEventArgs e);

	public enum SourceType { ConstantVoltage, ConstantCurrent }

	public class Resistor
	{
		// private fields (Datenfelder)

		double resistance = 1.0;
		double voltage = 1.0;
		double current = 1.0;
		double power = 1.0;
		double maxPower = 2.0;
		bool overload = false;
		SourceType source = SourceType.ConstantVoltage;

		// Constructor

		public Resistor()
		{
		}

		// Event

		public event OverloadEventHandler OverloadChanged;

		// Properties

		public double Resistance
		{
			get { return resistance; }
			set
			{
				if (value > 0.0) 
				{
					resistance = value;
					ApplyNewResistance();
				}
			}
		}

		public double Voltage
		{
			get { return voltage; }
			set
			{
				if (source == SourceType.ConstantVoltage)
				{
					voltage = value;
					ApplyNewVoltage();
				}
			}
		}

		public double Current
		{
			get { return current; }
			set
			{
				if (source == SourceType.ConstantCurrent)
				{
					current = value;
					ApplyNewCurrent();
				}
			}
		}

		public double MaxPower
		{
			get { return maxPower; }
			set
			{
				if (maxPower > 0.0)
				{
					maxPower = value;
					UpdatePower();
				}
			}
		}

		public SourceType Source
		{
			get { return source; }
			set { source = value; }
		}

		public double Power
		{ 
			get { return voltage*current; } 
		}

		public bool Overload 
		{ 
			get { return overload; } 
		}

		// private methods (operations)

		void ApplyNewResistance()
		{
			switch (source)
			{
				case SourceType.ConstantCurrent:
					voltage = resistance*current;
					break;
				case SourceType.ConstantVoltage:
					current = voltage/resistance;
					break;
			}
			UpdatePower();
		}

		void ApplyNewVoltage()
		{
			switch (source)
			{
				case SourceType.ConstantCurrent:
					break;
				case SourceType.ConstantVoltage:
					current = voltage/resistance;
					UpdatePower();
					break;
			}
		}

		void ApplyNewCurrent()
		{
			switch (source)
			{
				case SourceType.ConstantCurrent:
					voltage = resistance*current;
					UpdatePower();
					break;
				case SourceType.ConstantVoltage:
					break;
			}
		}

		void UpdatePower()
		{
			power = voltage*current;
			bool newOverload = power > maxPower;
			if (newOverload != overload)
			{
				overload = newOverload;
				// 
				if (OverloadChanged != null)
				{
					OverloadEventArgs e = new OverloadEventArgs(newOverload);
					OverloadChanged(this, e);
				}
			}
			overload = power > maxPower;
		}
	}
}
